% Fitting a multiport frequency response using the Complex Vector Fitting (CVF) method.
%
% Inputs:
%   1. Sample_test_points(Nc,Nc,Ns): matrix function to be fitted, where
%           Nc: dimension of H(s)
%           Ns: number of frequency samples
%   2. Frequency sample vector of size 1xNs
%   3. Initial poles
%   4. Options for the fitting
%   5. Passivity assessment and enforcement desired or not
%   6. Options for the passivity enforcement
%
% Outputs:
%   1. CVF model, including poles, residues and equivalent complex state-space representation
%   2. Frequency response of the CVF model
%   3. Maximum absolute model error (dB) between the scattering parameters and the corresponding CVF model
%   4. Matlab continuous-time state-space model (ss) equivalent to the computed CVF model
%
% NOTE: The use of this program is limited to NON-COMMERCIAL usage only.
% If the program code (or a modified version) is used in a scientific work,
% then reference should be made to the following:
%
%  [1] D. Spina, Y. Ye, D. Deschrijver, W. Bogaerts and T. Dhaene, Complex  
%  Vector Fitting Toolbox: a software package for the modeling and 
%  simulation of general linear and passive baseband systems, submitted to 
%  Electronics Letters, 2020
%    
%  [2] Y. Ye, D. Spina, D. Deschrijver, W. Bogaerts, and T. Dhaene, 
%  Time-domain compact macromodeling of linear photonic circuits via 
%  Complex Vector Fitting, Photonics Research, vol.: 7, issue: 7, 
%  pag.: 771-782, July 2019 
%
%  [3] Y. Ye, D. Spina, D. Deschrijver, W. Bogaerts and T. Dhaene,  
%  Efficient Time-Domain Modeling and Simulation of Passive Bandpass  
%  Systems, Proceedings of International Conference on Electromagnetics in 
%  Advanced Applications (ICEAA), Granada, Spain, 9-13 September 2019 



function [SER, bigHfit, err_vf, sys_tot]= Create_Rat_Mod_CVF(Samples_test_point,freq,init_poles,setting,BePassive,opts)



s_omega=2*pi*freq*1i; %%% Complex angular frequency samples


%%%% Fitting Baseband S-param

[SER,~,bigHfit,~]=VFdriver_complex(Samples_test_point,s_omega,init_poles,setting); %%% Computing the CVF model in pole-residue form and state-space representation
%     fit_original = bigHfit;  %%% Storing CVF Model response before passivity enforcement, if desired

if BePassive
    clear bigHfit
    [SER,bigHfit,~]=RPdriver_complex(SER,s_omega,opts);
end

sys_tot=ss(full(SER.A),full(SER.B),full(SER.C),full(SER.D));
err_vf=db(max(reshape(Samples_test_point-bigHfit,[],1)));
disp(['Modeling error: ' num2str(err_vf)])
disp(' ')

end

